This repository was archived by the owner on Jun 23, 2026. It is now read-only.
-
Notifications
You must be signed in to change notification settings - Fork 4
Expand file tree
/
Copy pathkernels.cpp
More file actions
291 lines (242 loc) · 7.64 KB
/
Copy pathkernels.cpp
File metadata and controls
291 lines (242 loc) · 7.64 KB
1
2
3
4
5
6
7
8
9
10
11
12
13
14
15
16
17
18
19
20
21
22
23
24
25
26
27
28
29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
/*
Copyright (c) 2015-present Advanced Micro Devices, Inc. All rights reserved.
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
#include <hip/hip_runtime.h>
#include <string>
#include <stdlib.h>
#include <cstdio>
#include <vector>
#include <cassert>
#include "kernels.h"
__global__ void LDS_bw(int numIter, float *dummy)
{
const uint32_t tid = threadIdx.x;
__shared__ uint8_t shmem[64];
if (tid == 0)
{
#pragma unroll
for (int i=0;i<63;i++)
shmem[i] = i+1;
shmem[63] = 0;
}
__syncthreads();
uint32_t index = tid;
#pragma unroll 64
for(uint32_t iter = 0; iter < numIter; iter++)
index = shmem[index];
dummy[tid] = (float )index;
}
using int32_16vec = __attribute__((__vector_size__(16 * sizeof(int)))) int;
using int32_8vec = __attribute__((__vector_size__(8 * sizeof(int)))) int;
using bf16_2vec = __attribute__((__vector_size__(1 * sizeof(__2i16)))) short;
using bf16_4vec = __attribute__((__vector_size__(2 * sizeof(__2i16)))) short;
using f32_16vec = __attribute__((__vector_size__(16 * sizeof(float)))) float;
using f16_2vec = __attribute__((__vector_size__(2 * sizeof(__2f16)))) float;
using f64_4vec = __attribute__((__vector_size__(4 * sizeof(double)))) double;
__global__ void mfma_i8(int iter, float *dummy)
{
// Output: 16 I32 registers
int32_16vec result = {0};
// MI100/MI200
#if defined(__gfx908__) or defined(__gfx90a__)
// Input: 1 I32 register
int a = threadIdx.x;
// CDNA1/2: v_mfma_i32_32x32x8i8 ops: 32x32x8x2 = 16384
for(int i = 0; i < iter; ++i)
{
result = __builtin_amdgcn_mfma_i32_32x32x8i8(a, a, result, 0, 0, 0);
}
// MI300 series
#else
// Input: 2 I32 registers
// builting mfma expects I64 input
long a = threadIdx.x;
// CDNA3: v_mfma_i32_32x32x16_i8 ops: 32x32x16x2 = 32768
for(int i = 0; i < iter; ++i)
{
result = __builtin_amdgcn_mfma_i32_32x32x16_i8(a, a, result, 0, 0, 0);
}
#endif
if (result[0] != 2*result[0])
{
dummy[0] = result[0];
}
}
/* Datatypes available for scale mfma f8f6f4 builtin
* 0 = fp8
* 1 = bf8
* 2 = fp6
* 3 = bf6
* 4 = fp4
*/
__global__ void mfma_f8f6f4(int iter, float *dummy, MX_DATAFORMATS datatype)
{
// MI350 series only
#if defined(__gfx950__)
// Input: 8 i32 registers
int32_8vec a;
a[0] = a[1] = a[2] = a[3] = a[4] = a[5] = a[6] = a[7] = threadIdx.x;
// Output: 16 F32 registers
f32_16vec result = {0};
// CDNA4: v_mfma_f32_32x32x64_f8f6f4 ops: 32x32x64x2 = 131072
switch (datatype)
{
case 1: // bf8 x bf8
for(int i = 0; i < iter; ++i)
{
result = __builtin_amdgcn_mfma_scale_f32_32x32x64_f8f6f4(a, a, result, 1, 1, 0, 0, 0, 0);
}
break;
case 2: // fp6 x fp6
for(int i = 0; i < iter; ++i)
{
result = __builtin_amdgcn_mfma_scale_f32_32x32x64_f8f6f4(a, a, result, 2, 2, 0, 0, 0, 0);
}
break;
case 3: // bf6 x bf6
for(int i = 0; i < iter; ++i)
{
result = __builtin_amdgcn_mfma_scale_f32_32x32x64_f8f6f4(a, a, result, 3, 3, 0, 0, 0, 0);
}
break;
case 4: // fp4 x fp4
for(int i = 0; i < iter; ++i)
{
result = __builtin_amdgcn_mfma_scale_f32_32x32x64_f8f6f4(a, a, result, 4, 4, 0, 0, 0, 0);
}
break;
default: // fp8 x fp8
for(int i = 0; i < iter; ++i)
{
result = __builtin_amdgcn_mfma_scale_f32_32x32x64_f8f6f4(a, a, result, 0, 0, 0, 0, 0, 0);
}
break;
}
if (result[0] != 2*result[0])
{
dummy[0] = result[0];
}
#endif
}
__global__ void mfma_f8(int iter, float *dummy)
{
// MI300 series only - note gfx940/gfx941/gfx942 only uses fnuz f8
#if defined(__gfx940__) or defined(__gfx941__) or defined(__gfx942__) or defined(__gfx950__)
// Input: 2 F32 registers
// builtin mfma expects double input
double a = threadIdx.x;
// Output: 16 F32 registers
f32_16vec result = {0};
// CDNA3: v_mfma_f32_32x32x16_fp8_fp8 ops: 32x32x16x2 = 32768
for(int i = 0; i < iter; ++i)
{
result = __builtin_amdgcn_mfma_f32_32x32x16_fp8_fp8(a, a, result, 0, 0, 0);
}
if (result[0] != 2*result[0])
{
dummy[0] = result[0];
}
#endif
}
__global__ void mfma_bf16(int iter, float *dummy)
{
// Output: 16 F32 registers
f32_16vec result = {0};
// MI100/MI200
#if defined(__gfx908__) or defined(__gfx90a__)
// Input: 1 F32 register
// builtin mfma expects 2 short registers
bf16_2vec a;
a[1] = a[0]= threadIdx.x;
// CDNA1/2: v_mfma_f32_32x32x4bf16 ops: 32x32x4x2 = 8192
for(int i = 0; i < iter; ++i)
{
result = __builtin_amdgcn_mfma_f32_32x32x4bf16(a, a, result, 0, 0, 0);
}
//MI300 series
#else
// Input: 2 F32 registers
// builting mfma expects 4 short registers
bf16_4vec a;
a[3] = a[2] = a[1] = a[0]= threadIdx.x;
// CDNA3: v_mfma_f32_32x32x8_bf16 ops: 32x32x8x2 = 16384
for(int i = 0; i < iter; ++i)
{
result = __builtin_amdgcn_mfma_f32_32x32x8bf16_1k(a, a, result, 0, 0, 0);
}
#endif
if (result[0] != 2*result[0])
{
dummy[0] = result[0];
}
}
__global__ void mfma_f16(int iter, float *dummy)
{
// Input: 2 F32 registers
f16_2vec a;
a[1] = a[0] = threadIdx.x;
//Output: 16 F32 registers
f32_16vec result = {0};
// CDNA2: v_mfma_f32_32x32x8f16 ops: 32x32x8x2 = 16384
// CDNA3: v_mfma_f32_32x32x8_f16
for(int i = 0; i < iter; ++i)
{
result = __builtin_amdgcn_mfma_f32_32x32x8f16(a, a, result, 0, 0, 0);
}
if (result[0] != 2*result[0])
{
dummy[0] = result[0];
}
}
__global__ void mfma_f32(int iter, float *dummy)
{
// Input: 1 F32 register
float a = threadIdx.x;
// Output: 16 F32 registers
f32_16vec result = {0};
// CDNA2: v_mfma_f32_32x32x2f32 ops: 32x32x2x2 = 4096
// CDNA3: v_mfma_f32_32x32x2_f32
for(int i = 0; i < iter; ++i)
{
result = __builtin_amdgcn_mfma_f32_32x32x2f32(a, a, result, 0, 0, 0);
}
if (result[0] != 2*result[0])
{
dummy[0] = result[0];
}
}
__global__ void mfma_f64(int iter, float *dummy)
{
// MI200 and above
#if not defined(__gfx908__)
// Input: 1 F64 register
double a = threadIdx.x;
// Output: 4 F64 registers
f64_4vec result = {0};
// CDNA2: v_mfma_f64_16x16x4f64 ops: 16x16x4x2 = 2048
// CDNA3: v_mfma_f64_16x16x4_f64
for(int i = 0; i < iter; ++i)
{
result = __builtin_amdgcn_mfma_f64_16x16x4f64(a, a, result, 0, 0, 0);
}
if (result[0] != 2*result[0])
{
dummy[0] = result[0];
}
#endif
}